#!/usr/bin/env python
import re
import sys, os, glob
import telnetlib
import unittest
import time

this_dir = os.path.abspath(os.path.dirname(__file__))

class JsshDriver:

    COMMAND_PROMPT = "\n> "

    def __init__(self, host=None, port=9997, telnetklass=telnetlib.Telnet):
        """Ctor
        	Ctor
        """
        self.host = host
        self.port = port
        self.tn = telnetklass(host,port)
        self.init()

    def open(self,host,port=9997):
        self.tn.close()
        self.host = host
        self.port = port
        self.tn.open(host,port)
        self.init()

    def init(self):
        if not self.tn.get_socket():
            return

        self.tn.read_until(JsshDriver.COMMAND_PROMPT)
        self.send_command("setProtocol('synchronous')")
	
    def send_command(self,command):
        self.tn.write(command + "\n")
        print (command + "\n")
        return self.tn.read_until(JsshDriver.COMMAND_PROMPT)

    def send_quit(self):
        self.tn.write("quit()\n")
        return self.tn.read_all()

class JsshTester:

    def __init__(self, host, port=9997, telnetklass=telnetlib.Telnet):
        self.browser = JsshDriver(host,port,telnetklass)

    def __del__(self):
        self.browser.send_quit()

    def get_variable_value(self,variable):
        jssh_response = self.browser.send_command ('print(' + variable + ')')

        m = re.compile(r"\[(?P<len>\d+)](?P<rest>.*)", re.DOTALL).search(jssh_response)

        return m.group('rest')[0:int(m.group('len'))]
        

    def get_innerHTML_from_URL(self,url):
        self.browser.send_command ('var browser = getWindows()[0].getBrowser()')

        if url:
            self.browser.send_command ('browser.loadURI("' + url + '")')

        self.browser.send_command ('var document = browser.contentDocument')
        self.browser.send_command ('var window = browser.contentWindow')
        return self.get_variable_value('document.documentElement.innerHTML')


class JsshTestCaseRunner(unittest.TestCase):

    def setUp(self):
        self.get_test_case_files()
        self.tester = JsshTester("localhost")
        self.tester.browser.send_command('var w = getWindows()[0]')
        self.tester.browser.send_command('var wc=w.content')

    def get_test_case_files(self):
        self.html_files = glob.glob("%s/test*html" % this_dir)

    def wait_for_var(self,variable_name):
        if "false" == self.tester.get_variable_value(variable_name):
            while "true" != self.tester.get_variable_value(variable_name):
                time.sleep(1)
        
    def run_if_function(self,function_name):
        return self.tester.browser.send_command('if (typeof ' + function_name + ' == \'function\') { ' + function_name +'(); }')

    def do_setup(self,filename):

        local_url = "file:///" + filename
        local_url = re.sub(r"\\", "/", local_url)

        self.tester.browser.send_command('wc.location="' + local_url + '";' )
        self.wait_for_var('wc.setupComplete')

        self.run_if_function('wc.setupTest')

    def do_runTest(self):
        self.run_if_function('wc.run_test')

        self.wait_for_var('wc.testComplete')

        self.failUnless("true" == self.tester.get_variable_value('wc.testResult'))
        self.run_if_function('wc.teardownTest')


    def testRunTest(self):
        for file in self.html_files:
            self.do_setup(file)
            self.do_runTest()


class PrefSetter:
  def __init__(self):
    self.tester = JsshTester("localhost")

  def setPreferences(self,val1,val2,val3):
    # val1 is the value of browser.tabs.loadDivertedInBackground
    # val2 is the value of browser.link.open_newwindow
    # val3 is the value of browser.link.open_newwindow.restriction
    b = self.tester.browser;
    b.send_command("var prefService = Components.classes['@mozilla.org/preferences-service;1'].getService(Components.interfaces.nsIPrefBranch);")
    b.send_command("prefService.setBoolPref('browser.tabs.loadDivertedInBackground', " + val1 + ");")
    b.send_command("prefService.setIntPref('browser.link.open_newwindow', " + val2 + ");")
    b.send_command("prefService.setIntPref('browser.link.open_newwindow.restriction', " + val3 + ");")

def main():
  prefSetter = PrefSetter();

  # We want to test all possible values of our preferences.  The preferences we
  # care about are:
  # pref1: browser.tabs.loadDivertedInBackground -- values are true/false
  # pref2: browser.link.open_newwindow -- values are 0, 2, 3.  "1" is the
  #                                       "current window" value; we might want
  #                                       to test it later... maybe.
  # pref3: browser.link.open_newwindow.restriction -- values are 0, 1, 2.

  # Notice that some combinations of these values will in fact lead to the same
  # results (for example, if newwindow.restriction is 1, it doesn't matter what
  # open_newwindow is).  But I'd rather not hardcode knowledge of that into
  # this test.

  for pref1 in ['true', 'false']:
    for pref2 in ['0', '2', '3']:
      for pref3 in ['0', '1', '3']:
        prefSetter.setPreferences(pref1, pref2, pref3)
        suite = unittest.makeSuite(JsshTestCaseRunner)
        # Make sure to use sys.stdout for errors, so that they interleave
        # properly with our print statements.
        unittest.TextTestRunner(sys.stdout).run(suite)

if __name__ == '__main__':
    main()
    
